@@ -1,12 +1,16 @@
+from difflib import unified_diff, Differ
 import json
 import sys
 import uuid
 import argparse
+import subprocess
+from contextlib import contextmanager
+from bashutils.colors import color_text
+import datadiff
 from mcloud.application import Application
 from mcloud.attach import AttachStdinProtocol
 from mcloud.config import YamlConfig
-from mcloud.sync.client import FileServerError
-from mcloud.sync.storage import get_storage, storage_sync
+from mcloud.sync import get_storage, rsync_folder
 from mcloud.util import txtimeout
 
 import re
@@ -19,6 +23,7 @@ from twisted.internet import defer, stdio
 from twisted.internet.defer import inlineCallbacks, CancelledError
 from twisted.internet.error import ConnectionRefusedError
 from mcloud import metadata
+import yaml
 
 
 def format_epilog():
@@ -125,6 +130,14 @@ class ApiRpcClient(object):
         self.current_client = None
         self.current_task = None
 
+    @contextmanager
+    def override_host(self, host):
+        back = self.host
+        self.host = host
+        yield
+        self.host = back
+
+
     @inlineCallbacks
     def _remote_exec(self, task_name, *args, **kwargs):
         from mcloud.remote import Client, Task
@@ -249,30 +262,27 @@ class ApiRpcClient(object):
     def print_app_list(self, data):
 
         x = PrettyTable(["Application name", "status", "cpu %", "memory", "Web", "Path"], hrules=ALL)
+        x.align = 'l'
         for app in data:
 
             app_cpu = 0.0
             app_mem = 0
             web = ''
 
+            service_status = ''
+
             for service in app['services']:
                 app_mem += int(service['memory'])
                 app_cpu += float(service['cpu'])
 
-            if app['running']:
-                app_status = app['status']
-                services_cpu_list = ('%.2f' % app_cpu) + '%'
-                services_memory_list = str(app_mem) + 'M'
+                if service['created']:
+                    service_status += ('^' if service['running'] else 'o')
+                else:
+                    service_status = 'x'
 
-            elif app['status'] == 'error':
-                app_status = 'ERROR: %s' % app['message']
-                web = app['message']
-                services_cpu_list = ''
-                services_memory_list = ''
-            else:
-                app_status = ''
-                services_cpu_list = ''
-                services_memory_list = ''
+            app_status = service_status
+            services_cpu_list = ('%.2f' % app_cpu) + '%'
+            services_memory_list = str(app_mem) + 'M'
 
             if app['status'] != 'error':
                 web_service_ = 'No web'
@@ -329,7 +339,9 @@ class ApiRpcClient(object):
                     raise ValueError('Can not parse application/service name')
 
         if not app and require_app:
-            raise ValueError('You should provide application name.')
+            app = os.path.basename(os.getcwd())
+            print('Using folder name as application name: %s\n' % app)
+            # raise ValueError('You should provide application name.')
 
         if service and app_only:
             raise ValueError('Command cannot be applied to single service')
@@ -423,10 +435,11 @@ class ApiRpcClient(object):
     @cli('Creates a new application', arguments=(
         arg('ref', help='Application and service name', default=None, nargs='?'),
         arg('path', help='Path', nargs='?', default='.'),
-        arg('--config', help='Config to use', nargs='?', default=None),
+        arg('--env', help='Application environment'),
+        arg('--config', help='Config to use', default=None),
     ))
     @inlineCallbacks
-    def init(self, ref, path, config=None, **kwargs):
+    def init(self, ref, path, config=None, env=None, **kwargs):
 
         app, service = self.parse_app_ref(ref, kwargs, app_only=True)
 
@@ -439,31 +452,72 @@ class ApiRpcClient(object):
         config.load(process=False)
 
         if self.host != '127.0.0.1':
-            success = yield self._remote_exec('init', app, config=config.export())
+            success = yield self._remote_exec('init', app, config=config.export(), env=env)
             if success:
                 yield self.sync(path, '%s@%s' % (app, self.host), no_remove=False, force=True, full=True)
         else:
             yield self._remote_exec('init', app, path=os.path.realpath(path), config=config.export())
 
-    @cli('Update application configuration', arguments=(
+    @cli('Application configuration', arguments=(
         arg('ref', help='Application and service name', default=None, nargs='?'),
-        arg('path', help='Path', nargs='?', default='.'),
+        arg('--set-env', help='Set application environment'),
         arg('--config', help='Config to use', nargs='?', default=None),
+        arg('--update', default=False, action='store_true', help='Update config'),
+        arg('--diff', default=False, action='store_true', help='Show diff only, do not update'),
     ))
     @inlineCallbacks
-    def update(self, ref, path, config=None, **kwargs):
-
+    def config(self, ref, diff=False, config=None, update=False, set_env=None, **kwargs):
         app, service = self.parse_app_ref(ref, kwargs, app_only=True)
 
-        if config:
-            config_file = os.path.expanduser(config)
+        app_config = yield self._remote_exec('config', app)
+
+        parser_env = set_env or app_config['env']
+
+        old_config = YamlConfig(source=unicode(app_config['source']), app_name=app, env=parser_env)
+        old_config.load(process=False)
+        from collections import OrderedDict
+        yaml.add_representer(unicode, yaml.representer.SafeRepresenter.represent_unicode)
+        yaml.add_representer(OrderedDict, yaml.representer.SafeRepresenter.represent_dict)
+        olds = yaml.dump(old_config.config, default_flow_style=False)
+
+        if not update and not diff and not set_env:
+            x = PrettyTable(["Name", "Value"], hrules=ALL, align='l', header=False)
+            x.align = "l"
+            x.add_row(['Config', olds])
+            x.add_row(['Environment', app_config['env']])
+            x.add_row(['Path', app_config['path']])
+            print(x)
+
         else:
-            config_file = os.path.join(path, 'mcloud.yml')
+            if config:
+                config_file = os.path.expanduser(config)
+            else:
+                config_file = os.path.join(app_config['path'], 'mcloud.yml')
 
-        config = YamlConfig(file=config_file, app_name=app)
-        config.load(process=False)
+            new_config = YamlConfig(file=config_file, app_name=app, env=parser_env)
+            new_config.load(process=False)
 
-        yield self._remote_exec('update', app, config=config.export())
+            if diff:
+                news = yaml.dump(new_config.config, default_flow_style=False)
+
+                if olds == news:
+                    print('Configs are identical.')
+                else:
+
+                    for line in unified_diff(olds.splitlines(1), news.splitlines(1)):
+                        if line.endswith('\n'):
+                            line = line[0:-1]
+                        if line.startswith('+'):
+                            print color_text(line, color='green')
+                        elif line.startswith('-'):
+                            print color_text(line, color='red')
+                        else:
+                            print line
+            else:
+                if set_env and not update:
+                    yield self._remote_exec('update', app, env=set_env)
+                else:
+                    yield self._remote_exec('update', app, config=new_config.export(), env=set_env)
 
 
     ############################################################
@@ -482,9 +536,10 @@ class ApiRpcClient(object):
     @cli('Start containers', arguments=(
         arg('ref', help='Application and service name', default=None, nargs='?'),
         arg('--init', help='Initialize applications if not exist yet', default=False, action='store_true'),
+        arg('--env', help='Application environment'),
     ))
     @inlineCallbacks
-    def start(self, ref, init=False, **kwargs):
+    def start(self, ref, init=False, env=None, **kwargs):
 
         app, service = self.parse_app_ref(ref, kwargs)
 
@@ -492,7 +547,7 @@ class ApiRpcClient(object):
             app_instance = yield self.get_app(app)
 
             if not app_instance:
-                yield self.init(app, os.getcwd())
+                yield self.init(app, os.getcwd(), env=env)
 
 
         data = yield self._remote_exec('start', self.format_app_srv(app, service))
@@ -674,25 +729,48 @@ class ApiRpcClient(object):
 
 
 
-
     ############################################################
     # File synchronization
     ############################################################
 
-    @cli('Push appliction volume application', arguments=(
-        arg('source', help='Push source'),
-        arg('destination', help='Push destination'),
-        arg('--no-remove', help='Disable remove files', default=False, action='store_true'),
-        arg('--force', help='Don\'t ask confirmation', default=False, action='store_true'),
-        arg('--full', help='Copy over all the files without diff', default=False, action='store_true'),
+    @cli('Push appliction volume', arguments=(
+        arg('volume', help='Volume name'),
+        arg('host', help='Host name', default=None, nargs='?'),
+        arg('--app', help='Application name', default=None),
+        arg('--path', help='Subpath to synchronize', default=None),
+        arg('--no-remove', help='Disable removing files on destination', default=False, action='store_true'),
+        arg('--update', help='Compare modification time and size, skip if match.', default=False, action='store_true'),
+        arg('--watch', help='Keep watching and uploading changed files', default=False, action='store_true'),
     ))
     @inlineCallbacks
-    def sync(self, source, destination, no_remove, force, full, **kwargs):
+    def push(self, volume, host, **kwargs):
+        app, service = self.parse_app_ref(None, kwargs, app_only=True)
+        config = yield self._remote_exec('config', app)
+        print config
 
-        src = get_storage(source)
-        dst = get_storage(destination)
 
-        yield storage_sync(src, dst, confirm=not force, verbose=True, remove=not no_remove, full=full)
+    @cli('Syncronize application volumes', arguments=(
+        arg('source', help='source'),
+        arg('destination', help='destination'),
+        arg('--path', help='Subpath to synchronize', default=None),
+        arg('--no-remove', help='Disable removing files on destination', default=False, action='store_true'),
+        arg('--update', help='Compare modification time and size, skip if match.', default=False, action='store_true'),
+        arg('--watch', help='Keep watching and uploading changed files', default=False, action='store_true'),
+    ))
+    @inlineCallbacks
+    def sync(self, source, destination, **kwargs):
+
+        src_type, src_args = get_storage(source)
+        dst_type, dst_args = get_storage(destination)
+
+        if src_type == 'remote' and dst_type == 'local':
+            yield rsync_folder(self, src_args, dst_args, options=kwargs)
+
+        elif src_type == 'local' and dst_type == 'remote':
+            yield rsync_folder(self, dst_args, src_args, reverse=True, options=kwargs)
+
+        else:
+            print('%s to %s is not supported' % (src_type, dst_type))
 
 
     ############################################################
