@@ -1,4 +1,9 @@
 import inspect
+import random
+import string
+from mcloud.container import PrebuiltImageBuilder
+from mcloud.service import Service
+from mcloud.sync import VolumeNotFound
 import os
 
 import re
@@ -38,7 +43,7 @@ class TaskService(object):
         pass
 
     @inlineCallbacks
-    def task_init(self, ticket_id, name, path=None, config=None):
+    def task_init(self, ticket_id, name, path=None, config=None, env=None):
         """
         Initialize new application
 
@@ -48,10 +53,6 @@ class TaskService(object):
         :return:
         """
 
-        print name
-        print path
-        print config
-
         app = None
         try:
             app = yield self.app_controller.get(name)
@@ -70,12 +71,12 @@ class TaskService(object):
             if not os.path.exists(path):
                 os.makedirs(path, 0700)
 
-        yield self.app_controller.create(name, {'path': path, 'source': config})
+        yield self.app_controller.create(name, {'path': path, 'source': config, 'env': env})
 
         defer.returnValue(True)
 
     @inlineCallbacks
-    def task_update(self, ticket_id, name, config=None):
+    def task_update(self, ticket_id, name, config=None, env=None):
         """
         Initialize new application
 
@@ -85,7 +86,7 @@ class TaskService(object):
         :return:
         """
 
-        yield self.app_controller.update_source(name, config)
+        yield self.app_controller.update_source(name, source=config, env=env)
 
         ret = yield self.app_controller.list()
         defer.returnValue(ret)
@@ -241,6 +242,27 @@ class TaskService(object):
             pass
 
 
+    @inlineCallbacks
+    def task_config(self, ticket_id, name):
+        """
+        Show application detailed status
+
+        :param ticket_id:
+        :param name:
+        :return:
+        """
+        app = yield self.app_controller.get(name)
+        config = yield app.load()
+
+        defer.returnValue({
+            'path': app.config['path'],
+            'env': app.get_env(),
+            'source': app.config['source'],
+            'hosts': config.get_hosts(),
+            'volumes': config.get_volumes(),
+        })
+
+
     @inlineCallbacks
     def task_status(self, ticket_id, name):
         """
@@ -343,6 +365,83 @@ class TaskService(object):
         return d
 
 
+    @inlineCallbacks
+    def task_sync_stop(self, ticket_id, app_name, sync_ticket_id):
+        s = Service()
+        s.app_name = app_name
+        s.name = '%s_%s_%s' % (app_name, '_rsync_', sync_ticket_id)
+
+        yield s.inspect()
+
+        if s.is_running():
+            self.task_log(ticket_id, 'Stopping rsync container.')
+            yield s.stop(ticket_id)
+
+        if s.is_created():
+            self.task_log(ticket_id, 'Destroying rsync container.')
+            yield s.destroy(ticket_id)
+
+
+    @inlineCallbacks
+    def task_sync(self, ticket_id, app_name, service_name, volume):
+
+        app = yield self.app_controller.get(app_name)
+
+        config = yield app.load()
+
+
+        s = Service()
+        s.app_name = app_name
+        s.name = '%s_%s_%s' % (app_name, '_rsync_', ticket_id)
+        s.image_builder = PrebuiltImageBuilder(image='modera/rsync')
+        s.ports = [873]
+
+        if service_name:
+
+            if not volume:
+                raise VolumeNotFound('In case of service name is provided, volume name is mandatory!')
+
+            services = config.get_services()
+
+            service_full_name = '%s.%s' % (service_name, app_name)
+            try:
+                service = services[service_full_name]
+
+                all_volumes = service.list_volumes()
+                if not volume in all_volumes:
+                    raise VolumeNotFound('Volume with name %s no found!' % volume)
+
+                volume_name = volume
+                s.volumes_from = service_full_name
+
+            except KeyError:
+                raise VolumeNotFound('Service with name %s was not found!' % service_name)
+
+        else:
+            s.volumes = [{
+                'local': app.config['path'],
+                'remote': '/volume'
+            }]
+            volume_name = '/volume'
+
+        s.env = {
+            'USERNAME': ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(32)),
+            'PASSWORD': ''.join(random.choice(string.ascii_lowercase + string.punctuation + string.digits) for _ in range(32)),
+            'ALLOW': '*'
+        }
+
+        yield s.start(ticket_id)
+
+        defer.returnValue({
+            'env': s.env,
+            'container': s.name,
+            'port': s.public_ports()['873'][0]['HostPort'],
+            'volume': volume_name,
+            'ticket_id': ticket_id
+        })
+
+
+
     @inlineCallbacks
     def task_start(self, ticket_id, name):
         """
@@ -630,9 +729,9 @@ class TaskService(object):
         """
         self.task_log(ticket_id, '[%s] Got response' % (ticket_id, ))
 
-        service = config.get_service(service_name)
-        if not service.is_running():
-            defer.returnValue('Not running')
+        service = config.get_service('%s.%s' % (service_name, name))
+        if not service.is_created():
+            defer.returnValue('Not created')
         else:
             if not service.is_inspected():
                 ret = yield service.inspect()
